@@ -1,7 +1,7 @@
 #
 #    DeltaFi - Data transformation and enrichment platform
 #
-#    Copyright 2021-2023 DeltaFi Contributors <deltafi@deltafi.org>
+#    Copyright 2021-2025 DeltaFi Contributors <deltafi@deltafi.org>
 #
 #    Licensed under the Apache License, Version 2.0 (the "License");
 #    you may not use this file except in compliance with the License.
@@ -22,6 +22,7 @@ from typing import List, NamedTuple
 from urllib.parse import urlparse
 
 import minio
+from minio.deleteobjects import DeleteObject
 
 BUCKET = 'storage'
 
@@ -55,24 +56,6 @@ class Segment(NamedTuple):
         return f"{self.did[:3]}/{self.did}/{self.uuid}"
 
 
-class ContentReference(NamedTuple):
-    segments: List[Segment]
-    media_type: str
-
-    def json(self):
-        return {
-            'segments': [segment.json() for segment in self.segments],
-            'mediaType': self.media_type
-        }
-
-    @classmethod
-    def from_dict(cls, content_reference: dict):
-        segments = [Segment.from_dict(segment) for segment in content_reference['segments']]
-        media_type = content_reference['mediaType']
-        return ContentReference(segments=segments,
-                                media_type=media_type)
-
-
 class ContentService:
     def __init__(self, url, access_key, secret_key):
         parsed = urlparse(url)
@@ -87,22 +70,24 @@ class ContentService:
         if not found:
             raise RuntimeError(f"Minio bucket {BUCKET} not found")
 
-    def get_bytes(self, content_reference: ContentReference):
+    def get_bytes(self, segments: List[Segment]):
         return b"".join([self.minio_client.get_object(BUCKET, segment.id(), segment.offset,
-                                                      segment.size).read() for segment in content_reference.segments])
+                                                      segment.size).read() for segment in segments])
 
-    def get_str(self, content_reference: ContentReference):
-        return self.get_bytes(content_reference).decode('utf-8')
+    def get_str(self, segments: List[Segment]):
+        return self.get_bytes(segments).decode('utf-8')
 
-    def put_bytes(self, did, bytes_data, media_type):
+    def put_bytes(self, did, bytes_data):
         segment = Segment(uuid=str(uuid.uuid4()),
                           offset=0,
                           size=len(bytes_data),
                           did=did)
-        content_reference = ContentReference(segments=[segment],
-                                             media_type=media_type)
         self.minio_client.put_object(BUCKET, segment.id(), io.BytesIO(bytes_data), len(bytes_data))
-        return content_reference
+        return segment
 
-    def put_str(self, did, string_data, media_type):
-        return self.put_bytes(did, string_data.encode('utf-8'), media_type)
+    def put_str(self, did, string_data):
+        return self.put_bytes(did, string_data.encode('utf-8'))
+
+    def delete_all(self, segments: List[Segment]):
+        delete_objects = [DeleteObject(seg.id()) for seg in segments]
+        return self.minio_client.remove_objects(BUCKET, delete_objects)
